#!/usr/bin/python3

import amulet
import telnetlib
import time


def check_config(a, b):
    config_string = a.file_contents('/etc/memcached.conf')
    # Parse the configuration file for the values sent in the deployment.
    for line in config_string.splitlines():
        print(line)
        if line.startswith('-x'):
            o = line.split()[1]
            break

    priv_addr_b, code = b.run('unit-get private-address')
    assert code == 0, "unit-get private-address failed, code %s" % code
    if o != priv_addr_b:
        print('(%s != %s)' % (o, priv_addr_b))
        amulet.raise_status(amulet.FAIL, msg=('Incorrect replica address '
                                              '(%s != %s)') % (o, priv_addr_b))


def check_replication(a, b, port):
    date_time = time.strftime("%F %r")
    string = 'memcached test %s' % date_time

    try:
        tn = telnetlib.Telnet(a.info['public-address'], port)
        command = 'set greeting 1 0 %d' % len(string)
        tn.write(command.encode() + b'\r\n')
        tn.write(string.encode() + b'\r\n')
        response = tn.read_until(b'STORED', 2)
        tn.close()

        tn = telnetlib.Telnet(b.info['public-address'], port)
        tn.write(b'get greeting\r\n')
        response = tn.read_until(b'END', 2)
        response = response.decode()
        index = response.find(string)

        if index != -1:
            print('Found %s in the greeting response.' % string)
        else:
            print(response)
            message = 'Did not find %s in the greeting from memcached' % string
            amulet.raise_status(amulet.FAIL, msg=message)
            tn.write(b'quit\n')

    except Exception:
        message = 'An error occurred communicating with memcached over telnet'
        amulet.raise_status(amulet.FAIL, msg=message)
    finally:
        try:
            tn.close()
        except NameError:
            pass


seconds = 1200
d = amulet.Deployment(series="trusty")
d.add('memcached', units=2)

configuration = {
    'repcached': True,
    'size': 512,
    'connection-limit': 128,
    'factor': 1.10,
    'tcp-port': 11211,
    'udp-port': 11213,
}

d.configure('memcached', configuration)
d.expose('memcached')

try:
    d.setup(timeout=seconds)
    d.sentry.wait(seconds)
except amulet.helpers.TimeoutError:
    message = 'The environment did not setup in %d seconds.' % seconds
    amulet.raise_status(amulet.SKIP, msg=message)
except:
    raise

units = (d.sentry['memcached'][0], d.sentry['memcached'][1])
memcached_port = configuration.get('tcp-port')

for memcached_unit in units:
    command = 'service memcached status'
    output, code = memcached_unit.run(command)

    # open memcache to be able to connect from this machine
    memcached_unit.run('ufw insert 1 allow {}'.format(memcached_port))

    if code != 0:
        message = 'The ' + command + ' returned %d.' % code
        print(output)
        amulet.raise_status(amulet.FAIL, msg=message)
    else:
        message = 'The memcached service is running.'
        print(output)
        print(message)

        memcached_address = memcached_unit.info['public-address']
        try:
            telnetlib.Telnet(memcached_address, memcached_port)
        except TimeoutError:  # noqa this exception only available in py3
            amulet.raise_status(amulet.FAIL, msg="Cannot connect to port %s" %
                                memcached_port)

primary, secondary = units

# if primary points to secondary
check_config(primary, secondary)

# if secondary points to primary
check_config(secondary, primary)

# if replication primary -> secondary works
check_replication(primary, secondary, memcached_port)

# if replication secondary -> primary works
check_replication(secondary, primary, memcached_port)
